"""Command-line interface."""
import click

from . import checks
from . import helpers


def _run_check(name, **kwargs):
    """Run a website check by name and log output.

    Args:
        name: The name of the website check to run.
        kwargs: keyword arguments to pass to the website check.

    Returns:
        None
    """
    event = {"check": name, "input": kwargs, "output": "fail"}

    try:
        result = getattr(checks, name)(**kwargs)
        event["output"] = "pass" if result else "fail"
        logger.info(event)
    except Exception as error:
        event["error"] = error
        logger.error(event)

    return None


@click.group(name="check")
@click.option("--debug/--no-debug", help="Toggle debug mode.")
@click.version_option()
def main(debug):
    """A simple python application for running checks against websites."""
    global logger
    log_level = "DEBUG" if debug else "INFO"
    logger = helpers.get_logger("website-checker", log_level)


# fmt: off
@main.command()
@click.option("urls", "--url", "-u", multiple=True, default=[None], show_default=True, help="The url to check.")  # noqa: B950
@click.option("--timeout", "-t", default=5, show_default=True, help="The HTTP request timeout in seconds.")       # noqa: B950
def network(urls, timeout):
    """Check website network connectivity."""
    for url in urls:
        _run_check("network", url=url, timeout=timeout)


# fmt: off
@main.command()
@click.option("urls", "--url", "-u", multiple=True, default=[None], show_default=True, help="The url to check.")  # noqa: B950
@click.option("--timeout", "-t", default=5, show_default=True, help="The HTTP request timeout in seconds.")       # noqa: B950
def health(urls, timeout):
    """Check website health."""
    for url in urls:
        _run_check("health", url=url, timeout=timeout)


# fmt: off
@main.command()
@click.option("urls", "--url", "-u", multiple=True, default=[None], show_default=True, help="The url to check.")   # noqa: B950
@click.option("--timeout", "-t", default=5, show_default=True, help="The HTTP request timeout in seconds.")        # noqa: B950
@click.option("--threshold", "-T", default=500, show_default=True, help="The latency threshold in milliseconds.")  # noqa: B950
def latency(urls, timeout, threshold):
    """Check website latency."""
    for url in urls:
        _run_check("latency", url=url, timeout=timeout, threshold=threshold)


def cli():
    """A wrapper function to pass auto_envvar_prefix to main entrypoint.

    The auto_envvar_prefix parameter needs to be passed to the script that is invoked.
    https://click.palletsprojects.com/en/7.x/options/#values-from-environment-variables
    """
    main(auto_envvar_prefix="CHECK")  # pragma: no cover


if __name__ == "__main__":
    cli()  # pragma: no cover



"""Helper functions."""
import datetime
import logging
import sys

from structlog import wrap_logger
from structlog.processors import JSONRenderer
from structlog.stdlib import add_logger_name
from structlog.stdlib import filter_by_level


def _add_timestamp(_, __, event_dict):
    """Add UTC ISO formatted timestamp to log event."""
    event_dict["timestamp"] = datetime.datetime.utcnow().isoformat()
    return event_dict


def get_logger(name, log_level):
    """Get JSON logger.

    Args:
        name: The logger name.
        log_level: The log level.

    Returns:
        A logger.
    """
    logging.basicConfig(
        level=log_level, stream=sys.stdout, format="%(message)s"
    )

    if log_level == "DEBUG":  # pragma: no cover
        processors = [
            filter_by_level,
            add_logger_name,
            _add_timestamp,
            JSONRenderer(indent=1, sort_keys=True),
        ]
    else:
        processors = [
            filter_by_level,
            add_logger_name,
            _add_timestamp,
            JSONRenderer(sort_keys=True),
        ]

    return wrap_logger(
        logging.getLogger(name),
        processors=processors,
    )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['website_checker']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0', 'requests>=2.25.0,<3.0.0', 'structlog>=20.1.0,<21.0.0']

entry_points = \
{'console_scripts': ['check = website_checker.cli:cli']}

setup_kwargs = {
    'name': 'website-checker',
    'version': '0.3.3a0',
    'description': 'A simple python application for running checks against websites.',
    'long_description': '[![Tests](https://github.com/aidanmelen/website-checker/workflows/Tests/badge.svg)](https://github.com/aidanmelen/website-checker/actions?workflow=Tests)\n[![Release](https://github.com/aidanmelen/website-checker/workflows/Release/badge.svg)](https://github.com/aidanmelen/website-checker/actions?workflow=Release)\n[![PyPI](https://img.shields.io/pypi/v/website_checker.svg)](https://pypi.org/project/website-checker/)\n[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/pre-commit/pre-commit)\n\n# website_checker\n\nA simple python application for running checks against websites.\n\n## Usage\n\n### Install\n\n```bash\n$ pipx install website-checker\n  installed package site-check 0.1.0, Python 3.9.0\n  These apps are now globally available\n    - check\ndone! ✨ 🌟 ✨\n\n# or install into system python with pip\n# pip install website-checker\n```\n\n### Example\n\nDisplay help message\n\n```text\n$ check --help\nUsage: check [OPTIONS] COMMAND [ARGS]...\n\n  A simple python application for running checks against websites.\n\nOptions:\n  --debug / --no-debug  Toggle debug mode.\n  --version             Show the version and exit.\n  --help                Show this message and exit.\n\nCommands:\n  health   Check website health.\n  latency  Check website latency.\n  network  Check website network connectivity.\n```\n\nSome examples\n\n```bash\n$ check network -u https://google.com -u https://blarg.com\n{"event": {"check": "network", "input": {"timeout": 5, "url": "https://google.com"}, "output": "pass"}, "logger": "website-checker", "timestamp": "2020-11-30T05:27:23.413281"}\n{"event": {"check": "network", "input": {"timeout": 5, "url": "https://blarg.com"}, "output": "fail"}, "logger": "website-checker", "timestamp": "2020-11-30T05:27:23.443994"}\n\n$ check health -u https://google.com\n{"event": {"check": "health", "input": {"timeout": 5, "url": "https://google.com"}, "output": "pass"}, "logger": "website-checker", "timestamp": "2020-11-30T05:27:49.413241"}\n\n$ check latency -u https://google.com\n{"event": {"check": "latency", "input": {"threshold": 500, "timeout": 5, "url": "https://google.com"}, "output": "pass"}, "logger": "website-checker", "timestamp": "2020-11-30T05:28:14.460530"}\n```\n\n### Docker\n\n```bash\n$ docker run --rm -it aidanmelen/website-checker health --url https://google.com\n{"event": {"check": "health", "input": {"timeout": 5, "url": "https://google.com"}, "output": "pass"}, "logger": "website-checker", "timestamp": "2020-11-30T05:00:23.444290"}\n```\n\n# License\n\nCheck out the [LICENSE](./LICENSE) for more information.\n\n# Credits\n\nCheck out the [CREDITS](./docs/CREDITS.md) for more information.\n',
    'author': 'Aidan Melen',
    'author_email': 'aidanmelen@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/aidanmelen/website-checker',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""URL checks."""
import datetime

import requests


def network(url, timeout):
    """Check the website network connectivity.

    Args:
        url: The URL to check.
        timeout: The request timeout.

    Returns:
        True on pass and False on failure.
    """
    try:
        _ = requests.head(url, timeout=timeout)
        return True
    except requests.ConnectionError:
        return False


def health(url, timeout):
    """Check the website health.

    Args:
        url: The URL to check.
        timeout: The request timeout.

    Returns:
        True on pass and False on failure.
    """
    return requests.get(url, timeout=timeout).ok


def latency(url, timeout, threshold):
    """Check the website latency.

    Args:
        url: The URL to check.
        timeout: The request timeout.
        threshold: The latency threshold.

    Returns:
        True on pass and False on failure.
    """
    with requests.get(url, timeout=timeout) as response:
        return response.elapsed < datetime.timedelta(milliseconds=threshold)
